#include <msp430x14x.h>
#include <in430.h>

/**************** Function prototypes ***********************************/
/*                                                                      */
void init_sys(void);                /* MSP430 Initialisation routine    */
void init_adc(void);                /* adc initialisation routine       */   
void delay(void);                   /* Software delay                   */
void complete(void);                /* Transmit / Receive ready         */
void convert(void);                 /* Do the conversions               */
void data_chk(void);                /* Check the conversions are OK     */
void display(void);                 /* Display the results on the LED   */
/*                                                                      */            
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Global variable declarations                                         */
/*                                                                      */
/************************************************************************/
int   channel;                      // Keeps track of the current channel
int   loop;                         // Keeps track of the number of conversions
int   adc_data[110];                // Storage for converted data
int   error;                        // Error count
int   Din;


/************************************************************************/
/*                                                                      */
/* main() variable declarations                                         */
/*                                                                      */
/************************************************************************/

#define     CS          0x01

void main(void)
{ 
init_sys();                   // Initialise the MSP430       

init_adc();                   // Initialise the adc - the interrupt    
                              //  that is generated from the conversion is ignored
convert();                    // Do conversions from each channel & store them in
   
data_chk();                   // Check each value stored in the array against what
                              //  is expected.
display();                    // Display the result on an LED                                                                                                                                                 
}

/************************************************************/
/* Prototype - init_sys                                     */
/*                                                          */
/* !NA init_sys                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype initialises the MSP430F149               */
/************************************************************/
void init_sys(void)
{
void setupXT2(void);    /* Local function prototype         */ 
void setupint(void);    /* Local function prototype         */
void setupports(void);  /* Local function prototype         */
void setupSPI(void);    /* Local function prototype         */    

setupXT2();       
setupint();
setupports();
setupSPI();
}
/************************************************************/
/* Prototype - setupXT2                                     */
/*                                                          */
/* !NA XT2_osc                                              */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the XT2 oscillator and tests     */
/*  that it has settled before moving on                    */
/************************************************************/      
void setupXT2 (void)

{
WDTCTL = WDTPW + WDTHOLD;           // Stop watchdog timer  
BCSCTL1&=XT2OFF;                    // Switch on the XT2 osc.
BCSCTL2|=SELM1+SELS;                // Select XT2 osc. for 
                                    // SMCLK & MCLK                                    
                                    // Test the osc. flag bit
      do {
         IFG1 &= ~OFIFG;            // Clear the OFIFG bit
         } while (OFIFG&IFG1);      //        
}
                             
/************************************************************/
/* Prototype - setupint                                     */
/*                                                          */
/* !NA setupint                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the P2 interrupt - this is       */
/*   the ADC interrupt                                      */
/************************************************************/      
void setupint (void)
{    
P2IFG &= ~BIT3;                     // Clear Interrupt Flag Register 
P2IES &= ~BIT3;                     // Set interrupt on Low-to-High transition
                                    // End Of Conversion (EOC)
}

/************************************************************/
/* Prototype - setupports                                   */
/*                                                          */
/* !NA setupports                                           */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the GPIO ports as appropriate    */
/************************************************************/      
void setupports (void)
{    
P1DIR = BIT0;                       // P1.0 output for LED
P1OUT &= ~BIT0;                     // Clear P1.0
P2DIR &= ~BIT3;                     // P2.3 input for interrupt
P3SEL = BIT3 + BIT2 + BIT1;         // Bits 3, 2 & 1 are assigned as SPI specific pins
P3DIR |= CS;                        // Set the CS bit as an output
P3OUT = CS;                         // De-assert CS for the adc - HIGH                             
}

/************************************************************/
/* Prototype - setupSPI                                     */
/*                                                          */
/* !NA setupSPI                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the P3 for communication via SPI */
/************************************************************/      
void setupSPI (void)

/************************************************************/
/* System definitions                                       */
/************************************************************/
#define     SPIen       0x40

{    
ME1 |= SPIen;                       // Module Enable - SPI
                                    // 
U0CTL &= ~SWRST;                    // Make sure the RESET bit is off                                   
U0CTL |= CHAR + SYNC + MM;          // USART0 module operation
                                    // CHAR = 1 => 8-bit data 
                                    // SYNC = 1 => SPI selected
                                    // MM = 1 => master mode,
                                    //     MSP is the master

U0TCTL |= SSEL0 + SSEL1 + STC;      // USART0 Tranmit control register
                                    // SSEL0 = 1 & SSEL1 = 1
                                    // => SMCLK is used for baud-rate generation
                                    // STC = 1 => 3-pin SPI mode selected
U0BR0  = 0x02;                         // Divide SMCLK by 4 => transfer clock
U0BR1  = 0x00;                     //
U0MCTL = 0x00;                   // Modulation control - not used. Ensure 
                                    // all these bits are reset                                                           
}

/************************************************************/
/* Prototype - init_adc                                     */
/*                                                          */
/* !NA init_adc                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the TLV2553 ADC                  */
/************************************************************/      
void init_adc (void)
{
P3OUT &= ~CS;                       // Assert CS LOW
      U0TXBUF = 0xBC;               // Send 1-byte test channel to the ADC
      complete();                   // Wait for the transfer to complete
      U0TXBUF = 0x00;               // Send 1-byte of data to the ADC - generates SCLK
      complete();                   // Wait for the transfer to complete
P3OUT |= CS;                        // De-assert CS HIGH
}

/************************************************************/
/* Prototype - complete                                     */
/*                                                          */
/* !NA complete                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype checks the UTXIFG0-bit                   */
/************************************************************/
void complete(void)
{
int i=0;
      do
      {
      IFG1 &=~ UTXIFG0;
      }
      while (UTXIFG0 & IFG1);      
}                                          

/************************************************************/
/* Prototype - delay                                        */
/*                                                          */
/* !NA delay                                                */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype gives a delay of around 1 second         */
/************************************************************/

void delay(void)
      {
      unsigned int i;
      for (i = 65000; i > 0; i--);
      }  

/************************************************************/
/* Prototype - convert                                      */
/*                                                          */
/* !NA convert                                              */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype controls which channel the ADC is        */ 
/*   converting and also the number of conversions per      */ 
/*   channel.                                               */
/************************************************************/      
void convert (void)
{

P3OUT &= ~CS;                             // Assert CS LOW
      U0TXBUF = 0xBC;                     //
      complete();                         //
      U0TXBUF = 0x00;                     //
      complete();                         //
      
P3OUT |= CS;                              // De-assert CS HIGH

P2IFG |= BIT3;                            // Set Interrupt Flag Register
      
                                          //*********** First interrupt-generating adc access *********
      
      _EINT();                            // Enable global interrupts 
            
      for(channel = 0; channel < 11; channel++)
      
      {           
                  for(loop = 0; loop < 10; loop++)                                
                  {
                  P2IE  |= BIT3;          // Interrupt requests from P2 are enabled 
                  }
      }              
}

/************************************************************/
/* Prototype - data_chk                                     */
/*                                                          */
/* !NA data_chk                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype reads each piece of converted data and   */
/*  checks to see if it's within a tolerance. The tolerance */
/*  in the program is + / - 5 counts                        */
/*  If there are any conversions that are not within the    */
/*  tolerance band the program will return a flag (error)   */
/*  to main()                                               */
/************************************************************/
void data_chk(void)
{
unsigned int test_val;
int k;
                             
      for(k = 0;k < 10; k++)
      {
      test_val = adc_data[k];            
       if (test_val > 0xE570) error = error + 1;
        else
         if (test_val < 0xE170) error = error + 1;
          else error = error + 0;
      }    
       
       for(k = 10;k < 20; k++)
       {                 
       test_val = adc_data[k];
        if (test_val > 0xC910) error = error + 1;
         else
          if (test_val < 0xC4C0) error = error + 1;
           else error = error + 0;
       }   
                
       for(k = 20; k < 30; k++)
       {
       test_val = adc_data[k];            
        if (test_val > 0xACE0) error = error + 1;
         else
          if (test_val < 0xA840) error = error + 1;
           else error = error + 0;
       }   
                
      for(k = 30; k < 40; k++)
      {
      test_val = adc_data[k];
       if (test_val > 0x9080) error = error + 1;
        else
         if (test_val < 0x8BE0) error = error + 1;
          else error = error + 0;
      }   
                
      for(k = 40; k < 50; k++)
      {
      test_val = adc_data[k];
       if (test_val > 0x7400) error = error + 1;
        else
         if (test_val < 0x6F60) error = error + 1;
          else error = error + 0;
      }   
               
      for(k = 50; k < 60; k++)
      {
      test_val = adc_data[k];
       if (test_val > 0x57A0) error = error + 1;
        else
         if (test_val < 0x5300) error = error + 1;
          else error = error + 0;
      }   
                
      for(k = 60; k < 70; k++)
      {
      test_val = adc_data[k];
       if (test_val > 0x3AD0) error = error + 1;
        else
         if (test_val < 0x3680) error = error + 1;
          else error = error + 0;
      }   
                
      for(k = 70; k < 80; k++)
      {
      test_val = adc_data[k];
       if (test_val > 0x1EC0) error = error + 1;
        else
         if (test_val < 0x1A20) error = error + 1;
          else error = error + 0;
      }
      
      for(k = 80;k < 90; k++)
      {
      test_val = adc_data[k];            
       if (test_val > 0xE570) error = error + 1;
        else
         if (test_val < 0xE170) error = error + 1;
          else error = error + 0;
      }    
       
       for(k = 90;k < 100; k++)
       {                 
       test_val = adc_data[k];
        if (test_val > 0xC910) error = error + 1;
         else
          if (test_val < 0xC4C0) error = error + 1;
           else error = error + 0;
       }   
                
       for(k = 100; k < 110; k++)
       {
       test_val = adc_data[k];            
        if (test_val > 0xACE0) error = error + 1;
         else
          if (test_val < 0xA840) error = error + 1;
           else error = error + 0;
       }   
                                      
}

/************************************************************/
/* Prototype - display                                      */
/*                                                          */
/* !NA display                                              */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype indicates the result of the test via the */
/*   LED on the MSP430 EVM:-                                */
/*   1. Solid LED, no flashes = test pass                   */
/*   2. Rapid flashing LED = only 1 failure from all samples*/
/*   3. Slow flashing LED = multiple failures from ADC      */
/************************************************************/
void display(void)
{
switch (error)
      {
      case 0:
            while(1)                // Repeat forever
            {
            P1OUT |= BIT0;          // Set P1.0
            }
            
      case 1:
            while(1)                // Repeat forever
            {
            P1OUT |= BIT0;          // Set P1.0
            }     
            
      case 2:
            while(1)                // Repeat forever
            {
            P1OUT |= BIT0;          // Set P1.0
            }
            
      case 3:
            while(1)                // Repeat forever
            {
            P1OUT |= BIT0;          // Set P1.0
            }                      

      default:
            while(1)
            {
            P1OUT |= BIT0;          // Set P1.0
            delay();
            P1OUT &= ~BIT0;         // Clear P1.0
            delay(); 
            }
      }          
}         
                        
/************************************************************/
/* Prototype - Interrupt Service Routine                    */
/*                                                          */
/* !NA read_adc                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This ISR reads the adc buffer                     */
/************************************************************/

interrupt [PORT2_VECTOR] void read_adc(void)
{      
int hi_byte;       
int lo_byte;
int element;
int   temp;

      {      
      P2IE  &= ~P2IE;                     // Interrupt requests from P2 are disabled   
      P3OUT &= ~CS;                       // Assert CS LOW                 
                  
      if(channel == 0 & loop == 0)
            {            
            U0TXBUF = 0x0C;                     //
            complete();                         //            
            U0TXBUF = 0x00;                     //
            complete();                         //                              
            
            delay();
            delay(); 
            
            U0TXBUF = 0x0C;                     //
            complete();                         // 
            hi_byte = U0RXBUF;
            hi_byte = hi_byte << 8;           
            U0TXBUF = 0x00;                     //
            complete();                         //            
            lo_byte = U0RXBUF;
            P3OUT |= CS;                              // De-assert CS HIGH    
            
            adc_data[0] = hi_byte + lo_byte;
            
            } 
            else if (loop == 9)
                  {
                  temp = channel + 1;
                  Din = temp << 4;
                  Din = Din | 0x1100;
                  U0TXBUF = Din;            
                  complete();
                  hi_byte = U0RXBUF;
                  hi_byte = hi_byte << 8;
            
                  U0TXBUF = 0x00;
                  complete();
                  lo_byte = U0RXBUF;
                                   
                  element = (10 * channel) + loop;
                  adc_data[element] = hi_byte + lo_byte;
                  }
                        else
                        {
                        Din = channel << 4;
                        Din = Din | 0x1100;            
                        U0TXBUF = Din;            
                        complete();
                        hi_byte = U0RXBUF;
                        hi_byte = hi_byte << 8;
            
                        U0TXBUF = 0x00;
                        complete();
                        lo_byte = U0RXBUF;
                                   
                        element = (10 * channel) + loop;
                        adc_data[element] = hi_byte + lo_byte;
                        }
      
      //delay();
      //delay(); 
      
      P3OUT |= CS;                        // De-assert CS HIGH                          
      }
}     
      

